;
;  C Startup.asm - Control Cross-C all-purpose startup code.
;
;  Copyright (c) 2000-2004 Softools, Inc.  All rights reserved.
;
;  Since this file is self-configuring using several assemble-time
;  prompts, it should be assembled separately for each application
;  program being developed.  Labels can be defined to bypass the
;  prompts - the WinIDE New Project Wizard in fact does this.
;
;  If a pre-startup module is used to initialize other hardware,
;  interrupt vectors, etc., the global entry point CSTART should
;  be jumped to from that module.  Otherwise, this module is placed
;  at location 0 and will handle the startup.  The state of interrupts
;  is NOT changed by this module.  Therefore, if a pre-startup module
;  turns them off, main () or this module should turn them back on.
;  The SP and a MMU and required Rabbit peripherals are initialized
;  by this module.
;
;  The following segments are used in the Rabbit Control Cross-C
;  environment.  Note that not all segments are necessarily used in
;  all applications.
;
;  Segment	Contains
;  ==================================================================
;  CODE 	Startup code, libraries and near compiled code.
;  STARTUP	Startup vector table used to dynamically run startup
;		  initialization code (must be conditionally included).
;  CONST	String literals, const objects and
;		  initializers for automatic aggregates.
;  FARCONST	Constants stored far.
;  DATA 	Initialized data area.	May be copied from ROM to RAM.
;  FARCODE	Code compiled as far functions in C.
;  FARCODE2	A TCP/IP module uses this.
;  FARDATA	Initialized far data area.  This is stored out of the
;		  logical address space.
;  BSS		Uninitialized data area.  This may be zeroed on startup.
;  FARBSS	Uninitialized far data area.  The startup code DOES NOT
;		  zero this space!
;  BSSZ 	Initialized by 0 data.	This may be zeroed on startup.
;  STACK	The amount reserved for the CPU stack
;		The remmaining area is left over and used for the dynamic heap.
;
;  --------------------------------------------------------------------------
;
;  The following assembly-time questions are asked to configure the
;    application:
;
;  We prompt for the CPU stack size.  This eliminates editing
;    this file just to change the stack size.
;
;  We prompt to find out if the system is battery backed.
;    If it is, then we do not zero-fill BSS and
;    We prompt if the system needs a power-up test for initialization.
;      If it does, then a variable stored in segment DATA is compared
;      to a known default value.  If the variable contains this value,
;      the initialization is skipped.  Otherwise, it is performed.
;    If not, segment BSS is never zero filled.
;
;  If STACK_SIZE is not supplied it will be prompted
;
Y	equ	1
N	equ	0
;
	.ifdef	STACK_SIZE
;
size	.equ	STACK_SIZE	; Application stack size?

	.ifdef	BATT_BACKED
	.error	"Battery backing RAM globally is no longer supported - use a new segment"
bb_sys	.equ	N		; Battery backed system?
	.endif

	.ifndef	INC_DEBUG
INC_DEBUG .equ	Y
	.endif
	.if	! INC_DEBUG
	.printx Building NO DEBUG region C startup (cannot debug in WinIDE)
	.endif

	.ifndef	GEN_CRC
GEN_CRC	.equ	N
	.endif

	.else
;
size::	.input	"Application CPU stack size? "
INC_DEBUG:.input "Include 2k debug kernel region (Y/N)? "
GEN_CRC:.input	"Add CRC table and runtime CRC check program (Y/N)? "
	.endif
;
	.include Rabbit.inc
;
__stack_size::	.equ	even size
;
	.cseg				; Define implicit segment ordering
	.defseg CONST
	.defseg STARTUP
	.dseg
	.defseg	FARCODE
	.defseg FARCONST
	.bseg
	.defseg BSSZ
	.defseg IDATA,data
	.defseg STACK,stack,align=2	; Define STACK segment
	.defseg FARBSS,data
	.seg	STACK
_stackMarker::
	.ds	4			; A 4-byte marker that can check for stack overruns
	.ds	__stack_size - 4
__stack::.equ	$			; Initial SP
;
	.cseg
	.phase	0
	jp	_skip			; This is at 0
inFlash:.ds	1			; Flash marker byte 3 (leave uninitialized)
;
	.org	08h
_reserved:.ds	2			; Do not use
_19200TC:.ds	1			; Bootstrap leaves 19200 TC here
_GCDR:	.ds	1			; And GCDR here.
_TAT4R:	.ds	1			; And TAT4R here.
	.org	0Eh
__CRC_location::
	.ds	2			; Store CRC here at E
;
;  Debug output and input vectors.  The __stdio FILE pointer
;  is set to these entry points.  In production code, printf
;  calls these per character doing nothing.  In debug mode,
;  the debug kernel redirects these to output to the WinIDE.
;
	.org	10h			; Debug in code
	ret
;
	.org	18h			; Debug out code
	ret
;
	.org	38h			; Debug test input code
__kbhit:xor	a,a
	ret
;
	.ifdef	_FLASH_DEBUG		; For debugging in flash (do not define!)
;
;  Install default vectors for debugging in flash.  These are
;  not loaded when running in RAM since the debug kernel is in RAM.
;
	.org	50h
	lcall	_dBreakpoint##
	.dl	0x4612B9EE		; Mark this program can flash debug

;
;  Include vector for debug kernel serial handler.
;
	.org	0C0h
	lcall	_dSerIntHandler##
;
	.endif
;
;
;  If INC_DEBUG is defined, cstart is built without a 2k hole normally
;  required for the debug kernel.  If you disable this, you cannot run
;  this in the debugger (even if the debug kernel is loaded).
;
;  For debugging in flash, this is still required if you want to switch back to RAM.
;
	.if	INC_DEBUG
;
	.org	800h			; 800h is the base above the debug kernel
;
	.else
;
	.org	60h+40			; Leave hole for system ID block
;
	.endif
	.dephase
;
_skip:	ljp	cstart			; Continue on with long jump
;
;  idBlock definition, defined as an address
;
_idBlock::	.equ	60h
__idBlockMac1::	.equ	60h+0Bh
__idBlockMac2::	.equ	60h+0Bh+6
;
;  This table defines the area of a program to CRC.  This is done
;  by SLINK automatically upon defining the table and location
;  to store the final CRC.  The code to CRC the program based on
;  this table is included in cstart and run if the label GEN_CRC
;  is non-0.  This makes no assumptions about the ordering and size
;  of segments.  The CRC is stored by SLINK (in all data formats
;  written) at __CRC_location.
;
	.if	GEN_CRC
__CRC_table::				; Must have this name to signal SLINK
	.dl	SGAT CODE+810h		; CRC segment CODE (minus first 16 bytes)
	.dl	SGSZ CODE-810h
	.dl	SGAT CONST		; CRC CONST
	.dl	SGSZ CONST
	.dl	SGAT DATA		; CRC DATA
	.dl	SGSZ DATA
	.dl	SGAT FARCODE		; CRC FARCODE
	.dl	SGSZ FARCODE
	.dl	SGAT FARCONST		; CRC FARCONST
	.dl	SGSZ FARCONST
	.dl	0			; Marks end of table
	.endif				; GEN_CRC
;
	.seg	FARCODE
cstart::ipset	3			; Being safe
	ljp	..next			; Set XPC in RAM debug
	.debug	{ cstart -1 211 cstart end
..next:	xor	a,a
	ld	iir,a
	ld	sp,__stack
;
;  Initialize the MMU SEGSIZE and STACKSEG to map logical data areas to their
;  physical addresses.  80h is removed for running in RAM since RAM is mapped to 0.
;  If RAM is located at C0000 (MB3CR) in your design, remove C0h instead of 80h.
;
	ld	a,(sgst BSS >> 8) & 0F0h
	ioi ld	(SEGSIZE),a
;
; Next commented line: experimental change to link BSS -> STACK at 80000+
;
;	ld	a,(sgat BSS >> 12) - 80h - (sgst BSS >> 12)
	ld	a,(sgat BSS >> 12) - (sgst BSS >> 12)
	ioi ld	(STACKSEG),a
;
	.if	INC_DEBUG
;
;  Test inFlash byte - will be 0 if the code is in RAM
;
	ld	a,(inFlash)		; Test for flash, skip init if not
	or	a			; Is non-0 in flash
	jp	z,..inRam
;
	.endif
;
;  Initialize Rabbit peripherals here.  This uses a table to
;  quickly load address and data to initialize the Rabbit processor.
;  Simply extend the table as needed.  The Watchdog is set to 2
;  seconds - plenty of time to get to main from here.
;
	ld	bc,..iTblSize		; Size
	ld	hl,..iTbl		; Table
..lp:   ld	e,(hl)			; Addr
	inc	hl
	ioi ldi
	jp	v,..lp
;
	.if	GEN_CRC
;
	lcall	$gencrc##
	jp	z,..cstart
;
;  CRC has failed.  Handling this is program dependent and must be defined in the user's program.
;
	jp	__crcFailedHandler##	; Handle a bad program CRC
;
	.else
;
	jp	..cstart
;
	.endif		; GEN_CRC
;
;  Flash only!  MBxCR and chip select initialization.
;
;  This defaults to having the Flash at 00000 and RAM at 80000.
;  Note: These do not update the shadow RAM locations.  If you need
;  to readback any registers used here, either initialize the RAM
;  (e.g. _srMMIDR is the MMIDR shadow location) or use the
;  shadow I/O functions in C to reinitialize them.
;  Note: MBxCR register shadows are initialized.
;
;  Defines for MBxCR registers.  Used here and to initialize the shadows.
;  MB0CR must be 0 wait states or the baud rate constants won't be
;  correct. In flash, this starts up in 0 wait states and later if the speed
;  would be too high doubling, it is changed to 1 wait state.  If 2
;  wait states are needed see set1ws below.
;
;  A 512k flash maps 2 quadrants to one CS (CS0).  This poses a problem
;  for a 2*256k system.  In this case, a runtime check is made and if
;  the ID block shows a non-512k device 0, quadrant 2 (40000) is mapped
;  to CS2 using MB1CRXval.  This setup matches all Z-World 2-flash boards.
;
;  Note: Be careful using wait states on early Rabbit 2000 processors.
;
;  You can define these in the project and override the default EQUs here.
;  You then must supply all 9 definitions or errors will result.
;
	.ifndef	MB0CRval
WS0	 	.equ	0C0h	; MBxCR: 0 ws (+1 wr)
WS1	 	.equ	080h	; MBxCR: 1 ws (+1 wr)
WS2	 	.equ	040h	; MBxCR: 2 ws (+1 wr)
WS3	 	.equ	000h	; MBxCR: 4 ws (+1 wr)
;
MB0CRval	.equ	WS0 | 8	; MB0CR: 0 ws, OE0, CS0 active, WP enabled
MB1CRval	.equ	WS0 | 8	; MB1CR: 0 ws, OE0, CS0 active, WP enabled
MB1CRXval	.equ	WS0 | 10; MB1CR: 0 ws, OE0, CS2 active, WP enabled
MB2CRval	.equ	WS0 | 5	; MB2CR: 0 ws, OE1, WE1, CS1 active
MB3CRval	.equ	WS0 | 5	; MB3CR: 0 ws, OE1, WE1, CS1 active
	.endif
;
..iTbl:
;
;  Set STACKSEG to 80000 - logical start of BSS
;  If RAM begins at C0000 change the 80 to C0.
;
	.db	STACKSEG,80h - (sgst BSS >> 12)
	.db	GCSR,	8	; GCSR: no periodic int, main osc no div
	.db	GCDR,	0	; Start not doubled
	.db	MMIDR,	0	; MMIDR: normal operation
	.db	GCM0R,	0	; Normal spread spectum
	.db	GCM1R,	80h	; Enabled
	.db	MB0CR,	MB0CRval | WS0	; Must have 0WS for startup!
	.db	MB1CR,	MB1CRval
	.db	MB2CR,  MB2CRval
	.db	MB3CR,	MB3CRval
	.db	TACSR,  001h	; TACSR: Enable timer A
	.db	SPCR,	080h	; SPCR: Disable SMODE pins
	.db	BDCR,	080h	; Disable RST 28
	.db     WDTCR,	5Ah	; WDTCR: 2 Seconds to get to main
;
..iTblSize=($-..iTbl)/2
;
;  This table of IO addresses will be zeroed.  This is to ensure
;  that uninitialized Rabbit peripherals and registers come up
;  consistently in Flash and RAM execution.
;
..zTbl:	.db	PADR, PBDR, PCDR, PDDDR, PDDR, PDCR, PDFR, PDDCR
	.db	PEDDR, PEDR, PECR, PEFR, TBCSR, TBCR, IB0CR, IB1CR
	.db	IB2CR, IB3CR, IB4CR, IB5CR, IB6CR, IB7CR, I0CR, I1CR
	.db	TAT1R, SBCR, SCCR, SDCR, GPSCR, BDCR, MECR, PFDR
	.db	PFCR, PFFR, PFDCR, PFDDR, PGDR, PGCR, PGFR, PGDCR
	.db	PGDDR, PWL0R, PWM0R, PWL1R, PWM1R, PWL2R, PWM2R, PWL3R
	.db	PWM3R, ICCSR, ICCR, ICT1R, ICT2R, ICS1R, ICS2R, QDCSR
	.db	QDCR, TAT1R, TACR, TAT2R, TAT8R, TAT3R, TAT9R, TAT10R
	.db	TAT5R, TAT6R, TAT7R, TBM1R, TBL1R, TBM2R, TBL2R, SECR
	.db	SAER, SFCR, SBER, SCER, SDER, SEER, SFER
;
..zTblSize=$-..zTbl
;
	.if	INC_DEBUG
;
;  Running in RAM at this point.  The folowing code can
;  be executed but only in RAM debugging - be careful
;  what is placed here - it won't run in the Flashed version.
;
..inRam:ipset	2			; So that serial break-outs work
	xor	a,a			; A NOP
	ld	(10h),a			; In RAM, remove RETs to allow printfs
	ld	(18h),a			; Remove this code to stub printf in RAM
	ld	(39h),a
	.endif
;
;  Zero Rabbit registers in zTbl
;
..cstart:
	xor	a,a
	ld	b,..zTblSize
	ld	hl,..zTbl
..zlp:	ld	e,(hl)
	ioi ld	(de),a
	inc	hl
	djnz	..zlp
;
;  Zero out the BSS data area.
;
	ld	hl, sgst BSS		; Logical start of BSS
	ld	bc, sgsz BSS		; Size of BSS
..zero:	ld	a,b
	or	c
	jr	z,..bssz
	ld	(hl),0			; 0 first byte
	inc	hl
	dec	bc
	jr	..zero
;
;  Zero out the BSSZ data area.  This is for data with initializers of
;  0 (e.g. int x = 0;)
;
..bssz: ld	hl, sgst BSSZ		; Start of BSSZ
	ld	bc, sgsz BSSZ		; Size of BSSZ
..loop: ld	a,b
	or	c
	jr	z,..data
	ld	(hl),0			; Zero BSSZ
	inc	hl
	dec	bc
	jr	..loop
;
;  Copy initialized data to segment IDATA.
;
..data:	ld	iy, word (sgat DATA)	; Phys start of DATA
	ld	e, sgat DATA >> 16
	ld	ix, sgst IDATA 		; Logical start of IDATA
	ld	bc, sgsz DATA		; Size of DATA
..copy:	ld	a,b
	or	c
	jr	z,..strt
	dec	bc
	ld	a,e			; No data
	ldp	hl, (iy)
	ld	(ix), l			; Copy
	inc	ix
	inc	iy
	ld	hl,iy
	bool	hl
	jr	nz,..copy
	inc	e
	jr	..copy
;
;  Read GCSR and store reset code
;
..strt:	ioi ld	a,(GCSR)
	rlca
	rlca
	and	a,3
	ld	(_resetCode),a
	ld 	a,1
	ld	(_srTACSR##),a		; Set TACSR (timer A is running)
	ld	(_srTAPR##),a		; Default value on a reset
	ld	a,0C0h
	ld	(_srPBDDR##),a		; Ditto
	add	a,a			; 80h
	ld	(_srSPCR##),a		; Ditto
	ld	a,(inFlash)		; Test for flash, skip init if not
	or	a			; Is non-0 in flash
	jr	nz,..flash
;
;  Initialize GCDR shadow as debug kernel left it and set other shadows
;
	ld	a,(_TAT4R)		; Init srTAT4R shadow
	ld	(_srTAT4R),a
	ld	a,(_GCDR)		; Init srGCDR shadow
	ld	(__GCDR),a		; Save for going in and out of doubling
	ld	(_srGCDR##),a
	ioi ld	(GCDR),a		; Restore doubler as on startup
	ld	a,0C0h			; Init srPCSR
	ld	(_srPCFR##),a
	ld	hl,_srMB0CR##		; Set MBxCR shadows
	ld	(hl),0C5h
	inc	hl
	ld	(hl),0C5h
	inc	hl
	ld	(hl),080h
	inc	hl
	ld	(hl),080h
	ld	a,(_19200TC)
	ld	(__serial19200TC),a	; Save 19200TC
	jr	..skDbl
;
;  Calculate 19200 TC.  This is required for setting baud rates and
;  to see if clock doubling can be enabled.
;  If SET_19200TC is defined to a value, the runtime speed check to get
;  the 19200 time constant is skipped and the defined value is used.
;
..flash:
	.ifndef	SET_19200TC
	call	SerialCalc19200TC
	.else
	ld	a,SET_19200TC
	.endif
	ld	de,__serial19200TC
	ld	(de),a     		; Save TC
	ld	hl,(07Bh)		; ID Block Flash 1 size in k
	ld	de,512			; Check for 512k
	sbc	hl,de			; ZF=1 if 512k
	ld	hl,_srMB0CR##
	ld	(hl),MB0CRval		; Set MBxCR shadows
	ld	e,MB0CR
	ioi ldi				; Fix CS0 - this was set to 0WS on startup
	nop
	ld	(hl),MB1CRval
	jr	z,..512			; Skip if 512k
	ld	e,MB1CR
	ld	(hl),MB1CRXval		; 256k MB1CR value
	ioi ldi
	dec	hl
..512:	inc	hl
	ld	(hl),MB2CRval
	inc	hl
	ld	(hl),MB3CRval
;
	.ifndef	DISABLE_CLOCK_DOUBLER
;
;  Enable clock doubling if possible.  This could enable 1 wait state
;  on all chip selects if the speed is above 30MHz
;  For low power operation, define DISABLE_CLOCK_DOUBLER or later in main
;  by writing a 0 to GCDR (using a shadow out).
;
	ld	hl,1			; Enable
	lcall	$enableClockDoubler##
	jr	..done
;
;  Here it's in RAM, if the
;  19200 TC indicates we need wait states, one is added, even
;  in RAM to keep the timing between RAM and flash the same.
;
..skDbl:cp	a,52			; 0 wait state cutoff (30MHz)
	jp	c,..done
	ld	hl,1			; 1 wait state (can be 2 or 4)
	push	hl
	ld	l,h			; 0 means set all MBxCRs
	push	hl
	lcall	$setWaitStates##
	add	sp,4
;
	.else	; No doubling
;
..flash:jr	..done
;
;  The debug kernel could have enabled doubling.  Turn it off if on.
;
..skDbl:
	bool	hl			; Disable
	ld	l,h
	lcall	$enableClockDoubler##
;
	.endif	; DISABLE_CLOCK_DOUBLING
;
..done:
;
;  Get function pointers in segment STARTUP and call each one.
;  This allows compile-time installment of startup initialization
;  which can be executed before main.  The order of call is the
;  order of the addresses in the output.  In a single module they
;  are guaranteed to occur in order.  Modules linked ahead of others
;  are guaranteed to execute their STARTUP functions first.
;  Included only if STARTUP_SEG is defined because not many users use
;  this and therefore the code is wasted.
;
	.ifdef	STARTUP_SEG
	ld	a,sgsz STARTUP / 2	; Number of entries
	or	a
	jr	z,..main
	ld	hl,sgst STARTUP
..glp:	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	push	hl
	push	af
	call	.callde
	pop	af
	pop	hl
	dec	a
	jr	nz,..glp
	.endif
;
;  Initialize xmem starting address and size.  This can be changed
;  if you want a fixed pool of a fixed size.  It's implemented here
;  with a simple memory sizing routine to maximize xmem in RAM and Flash.
;  This must do some work using the values in the MMU to find the real
;  address of memory since it varies in a RAM and flash enviroment.
;
..main: ld	ix,__xmemStart
	ioi ld	a,(STACKSEG)		; Get physical start of RAM
					; Add in logical adder
	add	a,(sgst stack + sgsz stack) >> 12
	bool	hl
	ld	l,a
	rr	hl
	rr	hl
	rr	hl
	rr	hl
	ld	h,0
	ld	(ix+2),hl		; Set MSW
	ld	(ix+10),hl		; Set heap pointer too
	ld	hl,(sgat stack + sgsz stack) << 4 & 0xFFF0
	rrca
	rr	hl
	rrca
	rr	hl
	rrca
	rr	hl
	rrca
	rr	hl
	ld	iy,hl
	ld	(ix),hl
	ld	(ix+8),hl
;
; Find the base of memory - in AIY
;
	ld	iy,0
	xor	a,a
..flp:	ldp	hl,(iy)
	ex	de,hl
	ld	hl,55AAh
	ldp	(iy),hl
	ldp	hl,(iy)
	ex	de,hl
	ldp	(iy),hl
	ld	hl,55AAh
	or	a,a
	sbc	hl,de
	jr	z,..ram
	ld	de,4000h
	add	iy,de           	; Add 16k
	adc	a,0
	jr	..flp
;
;  Find size of RAM - AIY is the start of RAM.
;
..safe:	.ds	2			; A safe place in case we hit this code
..ram:  ld	de,low ..safe		; Ensure AIY won't hit this code
	add	iy,de
	ldp	hl,(iy)			; Get word
	ex	de',hl			; Save it
	ld	hl,055AAh		; Store 55AA there
	ldp	(iy),hl
	ld	b,0			; Check for up to 256 4k pages
	ld	c,a			; C is base
	altd ld	hl,iy			; Save base
	jr	..sk4k			; Skip 4k or wrap test fails
..xlp:  ldp	hl,(iy)			; Save word
	ex	de,hl
	ld	hl,bc
	ldp	(iy),hl			; Store new word
	ex	af,af'			; Check for wrap
	ld	a,c
	exx
	ldp	hl',(hl)		; Check first page for wrap
	exx
	ld	a,0AAh
	cp	a,l
	jr	nz,..end
	ld	a,055h			; If we wrap we hit his
	cp	a,h
	jr	nz,..end		; Wrapped, then done (restores this word)
	ex	af,af'
	ldp	hl,(iy)
	ex	de,hl
	ldp	(iy),hl			; Restore word
	ex	de,hl
	or	a,a
	sbc	hl,bc			; Same?
	jr	nz,..end		; Jump if not
..sk4k:	ld	de,1000h
	add	iy,de           	; Add 4k
	adc	a,0
	cp	a,10h			; Reached end of space?
	jr	z,..end			; Done if so
	djnz	..xlp			; Try next 16k page
..end:	ex	de',hl			; Fix
	ld	a,c
	ldp	(iy),hl			; Restore first word
	ld	a,(inFlash)
	or	a,a
	ld	a,0
	jr	z,..cram
	sub	a,b			; Number of 4k pages of RAM
	push	af
	sub	a,((sgat stack + sgsz stack) - sgat bss + 0xFFF) / 1000h
	jr	..ok
..cram:	sub	a,b			; Number of 4k pages of RAM
	push	af                      ; Remove code used for RAM load
	sub	a,(sgat stack + sgsz stack + 0xFFF) / 1000h
..ok:	ld	c,a
	ld	b,0
	ld	de,1000h
	mul				; Bytes
	ld	(ix+6),hl		; High word
	ld	hl,bc
	ld	(ix+4),hl
	pop	af
	ld	b,0
	ld	c,a
	mul				; Amount of RAM
	ld	(ix+14),hl
	ld	hl,bc
	ld	(ix+12),hl
;
;  Check for a debug cable connection and enter debug kernel if detected.
;
	.ifdef	_FLASH_DEBUG		; For debugging in flash (do not define!)
;
	ld	a,(inFlash)		; Test for flash, skip init if not
	or	a			; Is non-0 in flash
	jr	z,..flDbg
	ioi ld	a,(SPCR)		; Check for debug cable
	and	40h			; SMODE0
	jr	z,..flDbg
	lcall	_dkStart##
..flDbg:
;
	.endif
;
;  Initialize __stdio to the output vectors.
;  Set the __stack_marker to known values to detect stack overruns.
;  Call main ().  Note, main must be far.  If it is declared near,
;  change the lcall to call and $main to _main.
;
	ld	hl,__debugStdio
	ld	(__stdio),hl
	ld	ix,_stackMarker
	ld	hl,5555h
	ld	(ix),hl
	add	hl,hl			; AAAA
	ld	(ix+2),hl
	ld	hl,_stackMarker + 4
	ld	de,_stackMarker + 5
	ld	bc,__stack_size - 5
	ld	(hl),55h
	ldir				; Fill stack with 55s
	lcall	$main## 		; Just halt if it returns
	ld	a,(inFlash)
	or	a,a			; If running in RAM do RST
	jr	nz,$                    ; If running in Flash - hang
_mainExit::
	rst	28h			; break into debugger
;
;  Here, the following routines are stubbed, since there is little that
;  can be done in an embedded system.  If you DO want to handle these
;  somehow, remove the public entry points here.
;
_abort::
	rst	28h			; break into debugger
;
;  kbhit routine for program to test for input from WinIDE.
;
_kbhit::
	jp	__kbhit
;
.callde:
	push	de			; Calls address in DE
	ret
;
;  Calculate the timer constant for 19200.
;  Returned in A.
;  This must be called with 0 wait states!
;
	.ifndef	SET_19200TC
SerialCalc19200TC:
	ld	a,5Ah
	ld	de,1FFFh	; mask for RTC bits
..wait: ioi ld	(RTC0R),a	; load RTC registers
	ioi ld	hl,(RTC0R)	; get LSW RTC
	and	hl,de		; mask off bits
	jr	nz,..wait	; wait bits 0-9 = 0
	ex	de,hl		; DE = count = 0
..loop: inc	de
	ld	b,49
	djnz	$
	ioi ld	(WDTCR),a
	ioi ld	(RTC0R),a
	ioi ld	hl,(RTC0R)	; get LSW RTC
	bit	4,h		; test bit 10
	jr    	z,..loop
	ld	a,d		; 19200 time constant
	ret
	.endif
end:	.debug	} end 748
;
;  Define variables defining free RAM available for the heap.
;  If the heap is moved, __heap_end must remain >= __heap_start.
;  If malloc/free are not used, then it doesn't matter.  These
;  must address memory that is always mapped in.  It's probable
;  but very important that the heap start is NOT 0.
;
;  We highly recommend against using malloc and free in an
;  embedded application.
;
;  Te define a head, set HEAP_SIZE in the Asm defines for cstart
;  or change the HEAP_SIZE definition below only when HEAP_SIZE
;  isn't already defined.
;
	.ifndef	HEAP_SIZE
HEAP_SIZE = 0
	.endif
;
__heap_start::
        dw      _heap
__heap_end::
        dw      _heap_end
;
        .bseg
_heap:	.ds	HEAP_SIZE
_heap_end:
;
	.bseg
;
;  Calculated time constant for TAT4R.  Use by serial libraries
;  to handle baud rates without regard to CPU speed.  If GCDR
;  is changed, this value must also be updated (going into
;  double clock mode means you must double this constant!)
;  This must be adjusted accordingly if GCDR is changed!
;
__serial19200TC::
	.ds	1
;
;  Reason for reset is stored here:
;    00 - Program restarted at 0
;    01 - Watchdog caused reset
;    03 - Reset line caused reset
;
_resetCode::
	.ds	1
;
;  Byte written to GCDR.  Can be used to go in and out of double speed.
;
__GCDR::
	.ds	1
;
;  Define _stdio here so we don't bring in fassign just to use this variable
;
__stdio::
	.ds	2
;
;  Define xmemory base, size, and heap pointer.  The xmem heap is sized at runtime
;  to allow the maximum xmem space in RAM and flash.  These must be kept together
;  and as defined.
;
__xmemStart::
	.ds	4
__xmemSize::
	.ds	4
__xmemHeap::
	.ds	4
__detectedRAM::
	.ds	4
;
;  An initialized FILE structure for initial printf redirection
;
	.dseg
__debugStdio:
	.dw	18h			; Output address
	.dw	10h			; Input address
	.dw	0			; Putback char
	.ds	2
;
	.if	INC_DEBUG
start	=	cstart
	.else
start	=	0xFFFF			; FFFF means no start (stops autorun)
	.endif
	.end	start
