#ifndef __TIMERB_LIB
#define __TIMERB_LIB
/* START LIBRARY DESCRIPTION *********************************************
timerb.lib

	Generic TIMERB.
	Support the following:
		1. periodic call to user routine every timer int
		2.	supports times from about 15us to 1500 seconds with 11.0592mHz xtal (22.1 clock)
		3. Note serial port baud rates <2700 effect timer, set up serial ports first
		Interrupts faster than 20,000/sec start to become too much overhead for use
	TimerBinit() must be called to initialize the ISR
		The routine uses the bios freq_divider unless XTAL_FREQ is defined
	Calls user-supplied function              void TimerBRoutine()
		if FAST_TIMERB is defined, only, hl,de,ix,a,f are saved in ISR and TimerBRoutine
			must be asm code that saves any regs it uses
Author: Scott Henion
Company: SHDesigns 				Web: http://www.shdesigns.org
END DESCRIPTION **********************************************************/

/* START FUNCTION DESCRIPTION ********************************************
TimerBInit                       <TIMERB.LIB>

SYNTAX:		int TimerBInit(int usec)

DESCRIPTION:
	Cause a user function TimerBRoutine() to be called every usec microseconds
	Notes:
		1. periodic call to user routine every timer int
		2.	supports times from about 20us to 1500 seconds with 11.0592mHz xtal (22.1 clock)
		3. Interrupts faster than 20,000/sec start to become too much overhead for use
	   4. TimerBinit() must be called to initialize the ISR
		5. Uses bios freq_divider for calcs
		6. If <100us, best to modify library asm isr. (inline code, no call to ext)
		7. Note serial port baud rates <2700 effect timer, set up serial ports first
		Calls user-supplied function              void TimerBRoutine()
		if FAST_TIMERB is defined, only regs: hl,de,ix,a,f are saved in ISR and
		 TimerBRoutine	MUST be asm code that saves any regs it uses. It must save
		 all regs if it calls a 'C' function.
Author: Scott Henion
Company: SHDesigns 				Web: http://www.shdesigns.org
RETURN VALUE:	non-0 if out of range
END DESCRIPTION **********************************************************/

int TimerBInit(unsigned long intspersec,char intlevel);

/* START FUNCTION DESCRIPTION ********************************************
TimerBRate                       <TIMERB.LIB>

SYNTAX:		float TimerBRate()

DESCRIPTION:
				Return actual ISR time in us;
RETURN VALUE:	float, usec per interrupt.
END DESCRIPTION **********************************************************/
float TimerBRate(void);


/* START FUNCTION DESCRIPTION ********************************************
TimerBUninit                       <TIMERB.LIB>

SYNTAX:		void TimerBUninit()

DESCRIPTION:  Stop the timer
RETURN VALUE:	None.
END DESCRIPTION **********************************************************/

 // this is how many time the Timerb isr will occur before the user function is called.
extern unsigned int _timerb_scale;

void TimerBUninit(void);

/*  the user function called from the library.

For the TimerBFast.lib - this is assumed assembly and only AF,HL and DE are saved.
	The user toutine should save any registers it needs.

For the TimerBSlow.lib - This function is assumed to be C and all registers are saved.

*/

void near TimerBRoutine();


#endif
